//
// A quicksort algorithm that uses stacks to avoid recursion.
// This one also uses the Median-of-three partitioning process to improve
// quicksort.
//
// Brad Merrill, May 22, 2001
//
using SortControls;

internal class Stack
  {
  private int[] stack;
  private int p;

  public Stack(int max)
    {
    stack = new int[max];
    p = 0;
    }

  public Stack() : this(100) { }
  public void Push(int v) { stack[p++] = v; }
  public int Pop() { return stack[--p]; }
  public bool IsEmpty { get { return (p == 0); } }
  }

internal class QSortMed3 : SortPanel
  {
  public override string sortAlg { get { return "QuickMedian3"; } }
  public override string sortLang { get { return "C#"; } }

  Stack sf = null;

  bool pauseTrue(int i, int j)
    {
    pause(i,j);
    return (true);
    }

  int partition(int[] a, int l, int r)
    {
    int i;
    int j;
    int v;
    int t;
    int m;

    m = l + (r-l)/2;	// establish the midpoint
    /*
     * the following is the 3-exchange method
     */
    if (a[l] > a[m])
      {
      t = a[m];
      a[m] = a[l];
      a[l] = t;
      pause(l,m);
      }
    if (a[l] > a[r])
      {
      t = a[l];
      a[l] = a[r];
      a[r] = t;
      pause(l,r);
      }
    if (a[m] > a[r])
      {
      t = a[m];
      a[m] = a[r];
      a[r] = t;
      pause(m,r);
      }
    /*
     * then exchange the one in the middle with r-1
     */
    t = a[m];
    a[m] = a[r-1];
    a[r-1] = t;
    pause(m,r-1);

    v = a[r-1]; // this was our median element

    /*
     * now run the partitioning on l+1 thru r-2
     */
    r = r-1;  // the test is i<r so just decr r by 1
    l = l+1;

    i = l-1;
    j = r;


    for (;;)
      {
      while (i < r && a[++i] <= v && pauseTrue(l,i) && !stopRequested)
	;
      while (j > l && a[--j] >= v && pauseTrue(r,j) && !stopRequested)
	;
      if (stopRequested)
	return (-1);
      if (i >= j)
	break;
      t = a[i];
      a[i] = a[j];
      a[j] = t;
      pause(i, j);
      }
    t = a[i];
    a[i] = a[r];
    a[r] = t;
    pause(i, r);
    return (i);
    }

  void QSortStacked(int[] a, int l, int r)
    {
    int i;

    for (;;)
      {
      while (r > l)
	{
	i = partition(a, l, r);
	if (stopRequested)
	  return;
	if (i-l > r-i)
	  {
	  sf.Push(l);
	  sf.Push(i-1);
	  pause(l,i);
	  l = i+1;
	  }
	else
	  {
	  sf.Push(i+1);
	  sf.Push(r);
	  pause(i,r);
	  r = i-1;
	  }
	}
      if (sf.IsEmpty)
	break;
      r = sf.Pop();
      l = sf.Pop();
      }
    }

  public override void sort(int[] a)
    {
    sf = new Stack(a.Length);
    QSortStacked(a, 0, a.Length-1);
    pause();
    }
  }

