<%@ Register TagPrefix="Acme" TagName="SourceCtrl" Src="SrcCtrl.ascx"%>

<%@ Import Namespace="System.Text" %>
<%@ Import Namespace="System.IO" %>

<html>
    <head>
        <link rel="stylesheet" href="style.css">
    </head>
    
    <script language="C#" runat="server">

        String width = "100%";
        String path;
        String font;
        String face;

        void Page_Load(Object sender, EventArgs e) {
           
            // Handle Font Information
            
            if (Page.IsPostBack == false) {
            
                font = Request.QueryString["font"];
       
                Hashtable ht1 = new Hashtable();
                ht1.Add("12", 0);
                ht1.Add("14", 1);
                ht1.Add("16", 2);
                ht1.Add("18", 3);
		ht1.Add("20", 4);
		ht1.Add("22", 5);
		ht1.Add("24", 6);
		ht1.Add("26", 7);
                
                if (font != null) {
                    FontSize.SelectedIndex = (int) ht1[font];
                }
            }

            if (font == null) {
                font = FontSize.SelectedItem.Value;
            }

            MySourceCtrl.FontSize = FontSize.SelectedItem.Value;


            // Handle .SRC Path 

            path = Request.QueryString["path"];

            if (path == null) {
                path = "../default.src";
            }


            // Handle file contents

            String fileName = Request.QueryString["file"];
            
            if (fileName != null) {
            
                // If an .src file, then redirect to that new location
                
		string fullPath = Path.Combine(Path.GetDirectoryName(path), fileName);
		
                if (fileName.ToLower().EndsWith(".src")) {
		      Response.Redirect("srcview.aspx?path=..\\" + fullPath);
                }
            
                // If a legal file to view, set source control viewer to display it
                MySourceCtrl.FileName = MapPathSecure(fullPath);
                Title.InnerHtml = fileName;
            }
            
            
            // Construct header list
            PopulateNavigationList(fileName, path);
        }
            
        void PopulateNavigationList(String fileName, String path) {
            
            FileStream fs = new FileStream(MapPathSecure(path), FileMode.Open, FileAccess.Read);
            StreamReader sr = new StreamReader(fs);
            
            char [] c1 = { ':' };
            char [] c2 = { ',' };
            String line; 
            
            while ( (line = sr.ReadLine()) != null ) {
                String [] list = line.Split(c1);
                if (list.Length > 1) {
                    HtmlTableRow row = new HtmlTableRow();

                    // Construct Group Label for Items
                    String sourcegroup = list[0];
                    
                    HtmlTableCell groupCell = new HtmlTableCell();
                    groupCell.InnerHtml = "<nobr><b>" + sourcegroup + ": </b></nobr>";
                    groupCell.Style.Add("padding-right", "10");
                    groupCell.Style.Add("color", "white");
                    row.Cells.Add(groupCell);
                    
                    // Construct List of Items
                    String [] sourcelist = list[1].Split(c2);
                    
                    if ((fileName == null) && (sourcelist.Length>0)) {
                        string dir = Path.GetDirectoryName(path);
                        fileName = MapPathSecure(Path.Combine(dir, ParseItem(sourcelist[0]).HRef));
                        MySourceCtrl.FileName = fileName;
                    }
                    
                    StringBuilder sb = new StringBuilder();

                    for (int i=0; i< sourcelist.Length; i++) {
                        ItemDetails item = ParseItem(sourcelist[i]);
                    
                        sb.Append("<a class='white' href='srcview.aspx?path=" + path + "&file=" + item.HRef  + "&font=" + font + "&face=" + face + "'>");                    
                        sb.Append(Path.GetFileName(item.Description));
                        sb.Append("</a>&nbsp;&nbsp; ");
                    }
                    
                    HtmlTableCell itemCell = new HtmlTableCell();
                    itemCell.InnerHtml = sb.ToString();
                    row.Cells.Add(itemCell);
                    
                    SourceTable.Rows.Add(row);
                }
            }
              
            fs.Close();
        } 
            
        public ItemDetails ParseItem(String item) {
            
            char [] c3 = { '|' };
           
            String [] itemParts = item.Split(c3);
            
            if (itemParts == null) {
                return null;
            } 
            
            ItemDetails itemDetails = new ItemDetails();
            
            if (itemParts.Length > 1) {
                itemDetails.Description = itemParts[0];
                itemDetails.HRef = itemParts[1];
            }
            else {
                itemDetails.Description = itemParts[0];
                itemDetails.HRef = itemParts[0];
            }
            
            return itemDetails;
        }    
            
        public class ItemDetails {
            public String Description;
            public String HRef;
        }   
            
        void Font_Select(Object sender, EventArgs e) {
            MySourceCtrl.FontSize = FontSize.SelectedItem.Value;
        }

    </script>
    
    <body>
        <form runat="server">

            <div class="SampleHeader" style="width:<%=width%>">

                <table cellpadding=5 cellspacing=0 background="grid_background.gif"  width="100%">
                    <tr>
                        <td class="TitleBar">
                            <div class="SampleTitle" >
                                <span id="Title" runat="server">Source Code Viewer</span>
                            </div>
                        </td>
                        <td class="TitleBar" align="right">
                            <span style="color:white; font-size:10pt; font-weight:700" >Size</span>&nbsp;
                            <asp:dropdownlist id="FontSize" autopostback="true" maintainstate="false" onselectedindexchanged="Font_Select" class="SampleSelect" runat="server">
                               <asp:listitem value="12">12 pt</asp:listitem>
                               <asp:listitem value="14">14 pt</asp:listitem>
                               <asp:listitem value="16">16 pt</asp:listitem>
                               <asp:listitem value="18">18 pt</asp:listitem>
                               <asp:listitem value="20">20 pt</asp:listitem>
                               <asp:listitem value="22">22 pt</asp:listitem>
                               <asp:listitem value="24">24 pt</asp:listitem>
                               <asp:listitem value="26">26 pt</asp:listitem>
                            </asp:dropdownlist>
                        </td>
                    </tr>
                </table>

                <table id="SourceTable" cellpadding="3" MaintainState="true" style="margin:5px;font: 8pt Verdana" runat="server"/>
            </div>

            <Acme:SourceCtrl id="MySourceCtrl" fontsize="18" runat="server" />
            
            <span style="color:red" id="ErrorMessage" runat="server"/>

        </form>
    </body>
</html>
