namespace Fib
{
/*
 * Fibonacci components driver
 * Brad Merrill 11-May-2001
 * 
 */
using System;
using System.Reflection;
using System.Text;
public class Driver
  {
  public const int MaxTerms = 4; // Maximum terms to calculate
  static string[] compNames = {"Fib_COBOL" };
  static public string[] langDesc;
  static public string[] Langs { get { return langDesc; } }
  static Type[] langType;
  static FibInstance[] langInst;

  static void init_components()
    {
    int len = compNames.Length;
    langDesc = new string[len];
    langInst = new FibInstance[len];
    langType = new Type[len];
    for (int i = 0; i < len; i++)
      {
      Assembly a = Assembly.Load(compNames[i]);
      langInst[i] = (FibInstance) a.CreateInstance(compNames[i]);
      langDesc[i] = langInst[i].Lang;
      }
    }

  static Driver()
    {
    rand = new Random();	// init randomizer
    init_vec();			// init the vector
    init_components();		// preload the assemblies
    }

  public static string CallFuncs(int ncalls, int nterms, int[] callee)
    {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < ncalls; i++) // iterate for N calls
      {
      sb.Append(call_lang(nterms, callee)); // call a lang component
      sb.Append(dump_vec(func_vec, func_lang)); // dump the current fib vector
      sb.Append(check_vec());		// check if any overflow
      }
    return sb.ToString();
    }

  static Random rand;
  public static void Main(string[] args) 
    {
    if (args.Length < 1) 
      {
      Console.WriteLine("Usage: Fib N\n\tWhere N is number of term functions");
      return;
      }
    int ncalls = Convert.ToInt32(args[0]);
    int nterms = 2;
    if (args.Length > 1)
      nterms = Convert.ToInt32(args[1]);

    int[] all = new int[langDesc.Length];
    for (int i = 0; i < langDesc.Length; i++)
      all[i] = i;
    Console.WriteLine(CallFuncs(ncalls, nterms, all));
    }

  static int[] func_vec;
  static string func_lang;

  /*
   * initialize the Fibonacci integer vector
   */
  static string init_vec()
    {
    func_lang = "Initial Vector";
    func_vec = new int[] { 1, 2 };
    return dump_vec(func_vec, func_lang);
    }

  /*
   * check for negative overflow, reset if needed
   */
  static string check_vec()
    {
    StringBuilder sb = new StringBuilder();
    for (int j = 0; j < func_vec.Length; j++)
      if (func_vec[j] < 0)
	{
	sb.Append(func_lang);
	sb.Append(": Negative overflow detected, Resetting Vector\n");
	sb.Append(init_vec());
	return sb.ToString();
	}
    return "";
    }

  /*
   * randomly call 1 of MaxTerms components
   */
  static string call_lang(int nterms, int[] callee)
    {
    int vec0 = func_vec[func_vec.Length-2];
    int vec1 = func_vec[func_vec.Length-1];

    int index = rand.Next(callee.Length);
    FibInstance f = langInst[callee[index]];

    try
      {
      func_vec = f.NextTerms(nterms, vec0, vec1);
      func_lang = f.Lang;
      }
    catch (OverflowException)	// if overflow
      {
      StringBuilder sb = new StringBuilder();
      sb.Append(langDesc[callee[index]]);
      sb.Append(": Positive Overflow detected, Resetting Vector\n");
      init_vec();
      return sb.ToString();
      }
    catch (Exception e)	// if any exceptions, just reset
      {
      Console.WriteLine("Error:"+e);
      Console.WriteLine("Exception detected, Resetting Vector\n");
      init_vec();
      }
    finally {}
    return "";
    }

  static string dump_vec(int[] v, string s)
    {
    StringBuilder sb = new StringBuilder();
    sb.Append(s);
    sb.Append(":");
    for (int i = 0; i < v.Length; i++)
      {
      if (v[i] < 0)
	break;
      sb.Append(" ");
      sb.Append(v[i]);
      }
    sb.Append("\n");
    return sb.ToString();
    }
  }
}
