namespace Fib
{
/*
 * Fibonacci components driver
 * Brad Merrill 11-May-2001
 * 
 */
using System;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

public class Driver
  {
  const int MaxTerms = 3; // Maximum terms to calculate
  //  static string[] compNames = { "Fib_CSharp", "Fib_VBNet" };
  public static string[] GetLangs(string path)
    {
    try
      {
      init_components(path);		// preload the assemblies
      }
    catch (Exception e)
      {
      throw new ApplicationException("Failed using "+path+":\n"+e);
      }      
    return langDesc;		// return the found assemblies
    }

  // this is to get the type that implements FibInterface
  static Type getFibType(Assembly a)
    {
    Type[] t = a.GetTypes();
    for (int j = 0; j < t.Length; j++)
      {
      Type[] f = t[j].GetInterfaces();
      for (int k = 0; k < f.Length; k++)
	if (f[k] == typeof(FibInterface))
	  {
	  return t[j];		// return the inheriting type
	  }
      }
    throw new ApplicationException(
      "Could not find Fib in assembly "+a.FullName);
    }

  static Type[] langTypes;
  static string[] langDesc;
  static FibInterface[] langInst;

  static void init_components(string path)
    {
    DirectoryInfo di = new DirectoryInfo(path);
    FileInfo[] files = di.GetFiles("Fib*.dll");

    if (files == null || files.Length == 0)
      throw new ApplicationException("No possible Fib components found");

    langTypes = new Type[files.Length];
    langInst = new FibInterface[files.Length];
    langDesc = new string[files.Length];

    for (int i = 0; i < files.Length; i++)
      {
      Match m = Regex.Match(files[i].Name, @"(\w+).dll");
      string name = m.Groups[1].ToString();
      Assembly a = Assembly.Load(name);
      langTypes[i] = getFibType(a);
      langInst[i] = (FibInterface) a.CreateInstance(langTypes[i].Name);
      langDesc[i] = langInst[i].Lang;
      }
    }

  static Driver()
    {
    rand = new Random();	// init randomizer
    init_vec();			// init the vector
    }

  public static string CallFuncs(int ncalls, int nterms, int[] callee)
    {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < ncalls; i++) // iterate for N calls
      {
      sb.Append(call_lang(nterms, callee)); // call a lang component
      sb.Append(dump_vec(func_vec, func_lang)); // dump the current fib vector
      sb.Append(check_vec());		// check if any overflow
      }
    return sb.ToString();
    }

  static Random rand;
  public static void Main(string[] args) 
    {
    if (args.Length < 1) 
      {
      Console.WriteLine("Usage: Fib N\n\tWhere N is number of term functions");
      return;
      }
    int ncalls = Convert.ToInt32(args[0]);
    int nterms = 2;
    if (args.Length > 1)
      nterms = Convert.ToInt32(args[1]);

    string[] all = GetLangs(".");
    int[] calls = new int[all.Length];
    for (int i = 0; i < all.Length; i++)
      calls[i] = i;
    Console.WriteLine(CallFuncs(ncalls, nterms, calls));
    }

  static int[] func_vec;
  static string func_lang;

  /*
   * initialize the Fibonacci integer vector
   */
  static string init_vec()
    {
    func_lang = "Initial Vector";
    func_vec = new int[] { 1, 2 };
    return dump_vec(func_vec, func_lang);
    }

  /*
   * check for negative overflow, reset if needed
   */
  static string check_vec()
    {
    StringBuilder sb = new StringBuilder();
    for (int j = 0; j < func_vec.Length; j++)
      if (func_vec[j] < 0)
	{
	sb.Append(func_lang);
	sb.Append(": Negative overflow detected, Resetting Vector\n");
	sb.Append(init_vec());
	return sb.ToString();
	}
    return "";
    }

  /*
   * randomly call 1 of MaxTerms components
   */
  static string call_lang(int nterms, int[] callee)
    {
    int vec0 = func_vec[func_vec.Length-2];
    int vec1 = func_vec[func_vec.Length-1];

    int index = rand.Next(callee.Length);
    FibInterface f = langInst[callee[index]];

    try
      {
      func_vec = f.NextTerms(nterms, vec0, vec1);
      func_lang = f.Lang;
      }
    catch (OverflowException)	// if overflow
      {
      StringBuilder sb = new StringBuilder();
      sb.Append(langDesc[callee[index]]);
      sb.Append(": Positive Overflow detected, Resetting Vector\n");
      init_vec();
      return sb.ToString();
      }
    catch (Exception e)	// if any exceptions, just reset
      {
      Console.WriteLine("Error:"+e);
      Console.WriteLine("Exception detected, Resetting Vector\n");
      init_vec();
      }
    finally {}
    return "";
    }

  static string dump_vec(int[] v, string s)
    {
    StringBuilder sb = new StringBuilder();
    sb.Append(s);
    sb.Append(":");
    for (int i = 0; i < v.Length; i++)
      {
      if (v[i] < 0)
	break;
      sb.Append(" ");
      sb.Append(v[i]);
      }
    sb.Append("\n");
    return sb.ToString();
    }
  }
}
